#include-once
#include <Array.au3>

Global Const $SYNTAX_SUB_START = 'sub '
Global Const $SYNTAX_FUNC_START = 'func '
Global Const $SYNTAX_COMMENT = ';'
Global Const $SYNTAX_STRING = '"'
Global Const $SYNTAX_LABEL = '#'
Global Const $SYNTAX_MACRO = '@'
Global Const $SYNTAX_MACRO_LINE = $SYNTAX_MACRO & 'line' ; beinhaltet die aktuelle Zeilennummer im ASM code


Global Const $REGEX_OPTIONAL_MINUS     = '[\-]?'
Global Const $REGEX_OPTIONAL_PLUS      = '[\+]?'
Global Const $REGEX_OPTIONAL_PLUSMINUS = '[\+\-]?'
Global Const $REGEX_ONE_TO_INF         = '+'
Global Const $REGEX_ZERO_TO_INF        = '*'

Global Const $REGEX_CHAR_VAR                = '[\w]' ; Alle erlaubten Zeichen für Namen
Global Const $REGEX_CHAR_STRING             = '[^,()#"+\-*\/<>=!\^&|~]' ; Alle verbotenen Zeichen innerhalb von Namen
Global Const $REGEX_CHAR_RELATION_OPERATORS = '[\<\>\=\!]' ; größer, kleiner, gleich
Global Const $REGEX_CHAR_MATH_OPERATORS     = '[\+\-\*\/\%\^\=\!]' ; z.B. a += b -> das + ist vorne

Global Const $REGEX_LINE_START = '\A(?i)'                       ; Zeilenanfang + case insensitive
Global Const $REGEX_LINE_END   = '$'                             ; Zeilenende


Global Const $REGEX_VAR_CORE = '[a-z_äöü]+[\wäöü]*'
Global Const $REGEX_NUM_CORE = '[+-]?(?:\d+(?:\.\d*)?)|(?:\.\d+)'
Global Const $REGEX_OP_CORE  = '[\+\-\*\/\%\^\=\!\<\>]'
Global Const $EX_VAR         = '(' & $REGEX_VAR_CORE & ')'
Global Const $EX_NUMBER      = '(' & $REGEX_NUM_CORE & ')'
Global Const $EX_VARORNUMBER = '(' & $REGEX_VAR_CORE & '|(?:' & $REGEX_NUM_CORE & '))'
Global Const $EX_VAROINTEGER = '(' & $REGEX_VAR_CORE & '|(?:' & '\d+' & '))'
Global Const $EX_FUNC        = '(' & $REGEX_VAR_CORE & ')' ; Buchstaben und Zahlen, muss mit Buchstabe beginnen
Global Const $EX_STRING      = '("[^"]+")' ; Im String darf alles stehen was kein Anführungsstrich ist.
Global Const $EX_MACRO       = '(@' & $REGEX_VAR_CORE & ')'
Global Const $EX_LABEL       = '(#' & $REGEX_VAR_CORE & ')'
Global Const $EX_ANYCALL     = $EX_FUNC & '\((.*)\)' ; func(...)
Global Const $EX_ANYFUNC     = $EX_VAR & '=' & $EX_ANYCALL ; var = func(...)
Global Const $EX_ANYEQUALITY = $EX_VAR & '=' & $EX_VARORNUMBER & '(' & $REGEX_OP_CORE & '{1,2}?)' & $EX_VARORNUMBER ; var = [+-]var ... [+-]var (1-3 Operatoren)
Global Const $EX_ANYMODIFY   = $EX_VAR & '(' & $REGEX_OP_CORE & '{1,3}?)' & $EX_VARORNUMBER
Global Const $EX_ANYINCDEC   = $EX_VAR & '((?:\+{2})|(?:\-{2}))'
Global Const $EX_GOTOINTVAR  = '(goto)\(' & '(' & $REGEX_VAR_CORE & '|(?:\d+))\)'
Global Const $EX_IF          = 'if\(' & $EX_VARORNUMBER & '(' & $REGEX_OP_CORE & '{1,2}?)' & $EX_VARORNUMBER & '\)'
Global Const $EX_ANYJUMPIF   = $EX_IF & $EX_GOTOINTVAR
Global Const $EX_IFSINGLE    = 'if\(' & $EX_VARORNUMBER & '\)(.*?)'
Global Const $EX_IFSINGLENEG = 'if\(!' & $EX_VARORNUMBER & '\)(.*?)'
Global Const $EX_ANYIF       = 'if\((.+?)\)(.*?)'
Global Const $EX_DEFARRAY    = $EX_VAR & '=Array\(' & $EX_VAR & '\)'
Global Const $EX_READARRAY   = $EX_VAR & '=' & $EX_VAR & '\[' & $EX_VAROINTEGER & '\]'
Global Const $EX_WRITEARRAY  = $EX_VAR & '\[' & $EX_VAROINTEGER & '\]' & '=' & $EX_VAROINTEGER

; var = array[int]
; var = array[var]
; array[int] = var
; array[var] = var



Global Const $PARAM_TYPE     = __Constants_PARAM_TYPE()

;~ ConsoleWrite($REGEX_LINE_START & $EX_WRITEARRAY & $REGEX_LINE_END & @CRLF)



Global Const $EX_RELATION2     = '\((' & $EX_VAR & ')(' & $REGEX_CHAR_RELATION_OPERATORS & '{1,2})(' & $EX_VAR & ')\)' ; (var1 != var2)
Global Const $EX_FOR_START     = 'for\([^,\n]+,[^,\n]+,[^,\n]+?\)'
Global Const $EX_FOR_END       = 'next'
Global Const $EX_LOOP_START    = 'loop'
Global Const $EX_LOOP_END      = 'endloop'
Global Const $EX_LOOP_EXIT     = 'exitloop'
Global Const $EX_LOOP_CONTINUE = 'continueloop'
Global Const $EX_IF_LEXIT      = 'if' & $EX_RELATION2 & $EX_LOOP_EXIT
Global Const $EX_IF_LCONTINUE  = 'if' & $EX_RELATION2 & $EX_LOOP_CONTINUE
Global Const $EX_SUB_START0    = $SYNTAX_SUB_START & $EX_VAR & '\(\)'  ; Beginn Subroutine ohne Parameter
Global Const $EX_FUNC_START0   = $SYNTAX_FUNC_START & $EX_VAR & '\(\)' ; Beginn Funktion ohne Parameter
Global Const $EX_ANY_CALL0     = $EX_VAR & '\(\)'                      ; call ohne alles.


Global Const $LABEL_FOR_START   = $SYNTAX_LABEL & '_for_'
Global Const $LABEL_FOR_END     = $SYNTAX_LABEL & '_next_'
Global Const $LABEL_IF_ONELINER = $SYNTAX_LABEL & '_olif_'
Global Const $LABEL_IF_START    = $SYNTAX_LABEL & '_if_'
Global Const $LABEL_SUB_START   = $SYNTAX_LABEL & '_sub_'
Global Const $LABEL_SUB_END     = $SYNTAX_LABEL & '_endsub_'

Global Const $VAR_SUB_RETURNPOINTER = '_ptr_subret_'
Global Const $VAR_SUB_RETURNVALUE   = '_val_subret_'

Global Const $DISPLAY_SIZE_SMALL = 80
Global Const $DISPLAY_SIZE_LARGE = 176





Func __Constants_PARAM_TYPE()
	Local $_[]
	$_['var']         = $EX_VAR
	$_['number']      = $EX_NUMBER
	$_['varornumber'] = $EX_VARORNUMBER
	$_['func']        = $EX_FUNC
	$_['string']      = $EX_STRING
	$_['macro']       = $EX_MACRO
	$_['label']       = $EX_LABEL
	Return $_
EndFunc